/*
 *
 *  Copyright (C) 1999, Institute for MicroTherapy
 *
 *  This software and supporting documentation were developed by
 *
 *    University of Witten/Herdecke
 *    Department of Radiology and MicroTherapy
 *    Institute for MicroTherapy
 *    Medical computer science
 *    
 *    Universitaetsstrasse 142
 *    44799 Bochum, Germany
 *    
 *    http://www.microtherapy.de/go/cs
 *    mailto:computer.science@microtherapy.de
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND THE INSTITUTE MAKES  NO 
 *  WARRANTY REGARDING THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY
 *  OR FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES 
 *  OR ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY 
 *  AND PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Author :      $Author: Schroeter $
 *  Last update : $Date: 1999/05/14 09:33:09 $
 *  Revision :    $Revision: 1.1.1.1 $
 *  State:        $State: Exp $
*/

package J2Ci;

/** 
 * A <em>jDVInterface</em> represents the relating C++-Class DVInterface
 * in Java.
 *
 * @author 	Andreas Schrter
 */
public class jDVInterface
{    
    /**
     * Constructor, creates a relating C++-Object.
     * @param configfile s String defining the name of the config file.
    */
    public jDVInterface(String configfile)
    {
        createObjOfDVInterface (configfile);
    }
        
        
    /**
     * Destructor, removes *this* relating C++-Object out of memory
    */
    protected void finalize ()
    {        
        removeObjOfDVInterface ();        
    }
    
    // -------------------------------- Methods for C++-Class Binding
    
    /**
     * Address of relating C++-Object [for access to the DLL].
     * Never change manually!
    */
    private long cppClassAddress = (long) 0; // never change!
    
    
    /**
     * Creates a C++-Object and write its address in field "cppClassAddress"
     * 
     * @see J2Ci.jDVInterface#cppClassAddress
    */
    private native void createObjOfDVInterface (String indexfolder);
    
       
    
    /**
     * Removes a C++-Object with the address of field cppClassAddress out of memory.
     * @see J2Ci.jDVInterface#cppClassAddress
    */
    private native void removeObjOfDVInterface ();
    
     
       
    // --------------------------- native Methods ------------------------------

    /**
     * Loads an image/instance
     * @param filename filename of the instance
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int loadImage(String filename);
    
    
    /**
     * Loads an image from a series/study
     * @param studyUID    UID of Study
     * @param seriesUID   UID of Series
     * @param instanceUID UID of Image
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int loadImage(String studyUID, String seriesUID, String instanceUID);
    
    
    /**
     * Loads a PresentationState for an image
     * @param pstName Name of PresentationStates
     * @param imgName Name of Image
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int loadPState(String pstName, String imgName);
    
    
    /**
     * Loads a PresentationState for a study/series
     * @param studyUID    UID of Study
     * @param seriesUID   UID of Series
     * @param instanceUID UID of Image
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int loadPState(String studyUID, String seriesUID, String instanceUID);
    

    /**
     * Saves a PresentationState
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int savePState();
    
    
    /**
     * Saves a PresentationState as...
     * @param filename Filename of PresentationState
     * @param explicitVR selects the transfer syntax explicit/implicit VR little endian.
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int savePState(String filename, boolean explicitVR);

       
    /**
     * Gets the current PresentationState-Object
     * @return jDVPresentationState-Object
    */
    public jDVPresentationState getCurrentPState()
    {
        return new jDVPresentationState (getCurrentPStateN());
    }
   
    private native long getCurrentPStateN();
    
    
    /**
     * Releases the database.
     * Removes any shared or exclusive lock on the database. This method should be 
     * called when a database transaction is finished (i.e. reading studies, ...).
     * As long as a lock exists on the db, no other application can add new images
     * to the db.
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int releaseDatabase();
    
    
    /**
     * Gets the amount of studies.
     * @return amount of studies.
    */
    public native int getNumberOfStudies();
    
    
    /**
     * Select a study.
     * @param idx Index of study
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int selectStudy(int idx);
    
    
    /**
     * Gets the UID of the selected study
     * @return UID
    */
    public native String getStudyUID();
    
    
    /**
     * Gets the desciption of the selected study.
     * @return Description
    */
    public native String getStudyDescription();
    
    
    /**
     * Gets the state of the selected study.
     * @return Value (from jDVIFhierarchyStatus).
    */
    public native int getStudyStatus();
    

    /**
     * Gets the number of series.
     * @return Number of series.
    */
    public native int getNumberOfSeries();
    
    
    /**
     * Selects a series.
     * @param idx Index of series
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int selectSeries(int idx);
    
    
    /**
     * Gets the UID of the selected series.
     * @return UID
    */
    public native String getSeriesUID();
    
    
    /**
     * Gets the state of the selected series.
     * @return Value from jDVIFhierarchyStatus
    */
    public native int getSeriesStatus();
    

    /**
     * Checks if this Series contains a PresentationState.
     * @return true, if yes.
    */
    public native boolean isPresentationStateSeries();
    
    
    /**
     * Gets the number of images.
     * @return Number of images.
    */
    public native int getNumberOfInstances();
    
    
    /**
     * Selects an image.
     * @param idx Index of image.
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int selectInstance(int idx);
    
    
    /**
     * Gets the UID of the selected image.
     * @return UID
    */
    public native String getInstanceUID();
    
    
    /**
     * Gets the modality of the selected image.
     * @return The modality.
    */
    public native String getModality();
    
    
    /**
     * Gets the state of the selected image.
     * @return Value fom jDVIFhierarchyStatus
    */
    public native int getInstanceStatus();
    
    
    /**
     * Checks if the image is a PresentationState.
     * @return true, if yes.
    */
    public native boolean isPresentationState();
    
    
    /**
     * If the image is reviewed, the state of the database can be changed from "isNew" to 
     * "isNotNew".
     * 
     * @param studyUID    UID of Study
     * @param seriesUID   UID of Series
     * @param instanceUID UID of Image
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int instanceReviewed(String studyUID, String seriesUID, String instanceUID);
    
    
    /**
     * Removes an image
     * @param studyUID    UID of Study
     * @param seriesUID   UID of Series
     * @param instanceUID UID of Image
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int deleteInstance(String studyUID, String seriesUID, String instanceUID);
    
    
    /**
     * Removes a series.
     * @param studyUID    UID of study
     * @param seriesUID   UID of series
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int deleteSeries(String studyUID, String seriesUID);
    
    
    /**
     * Removes a study
     * @param studyUID    UID of Study
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int deleteStudy(String studyUID);
    
    
    /**
     * Sends an IOD to another application or modality
     * @param targetID    ID of target
     * @param studyUID    UID of Study
     * @param seriesUID   UID of Series
     * @param instanceUID UID of Image     
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int sendIOD(String targetID, String studyUID, String seriesUID, String instanceUID);
    
    
    /**
     * Gets the number of possible targets.
     * @return Number of targets.
    */
    public native int getNumberOfTargets();
    
    
    /**
     * Gets a target with at an index.
     * @param idx Index of target
     * @return ID of the target.
    */
    public native String getTargetID (int idx);
    
    
    /**
     * Gets the description of a target at an index.
     * @param idx Index of target.
     * @return Target description
    */
    public native String getTargetDescription (int idx);
    
    
    /**
     * Gets a target description of a target with a known ID.
     * @param descr ID of target
     * @return Target description
    */
    public native String getTargetDescription (String targetID);
    
    
    /**
     * Returns the study date of a study which is selected by "selectStudy".
     * @return study date
    */
    public native String getStudyDate ();
    
    
    /**
     * Returns the study time of a study which is selected by "selectStudy".
     * @return study time
    */
    public native String getStudyTime ();
    
    
    /**
     * Returns the referring physicians name of a study which is selected by "selectStudy".
     * @return physicians name
    */
    public native String getReferringPhysiciansName ();
    
    
    /**
     * Returns the accession number of a study which is selected by "selectStudy".
     * @return accession number
    */
    public native String getAccessionNumber ();
    
    
    /**
     * Returns the name of a physician reading a selected study by "selectStudy".
     * @return name of physician
    */
    public native String getNameOfPhysiciansReadingStudy ();
    
    
    /**
     * Returns the series number of a series which is selected by "selectSeries"
     * @return series number
    */
    public native String getSeriesNumber ();

    
    /**
     * Returns the series date of a series which is selected by "selectSeries".
     * @return series date
    */
    public native String getSeriesDate ();
    
    
    /**
     * Returns the series time of a series which is selected by "selectSeries".
     * @return series time
    */
    public native String getSeriesTime ();    
    
    
    /**
     * Returns the series description of a series which is selected by "selectSeries".
     * @return series description
    */
    public native String getSeriesDescription ();
    
    
    /**
     * Returns the series performing physicians name of a series which is selected 
     * by "selectSeries".
     * @return physician name
    */
    public native String getSeriesPerformingPhysiciansName ();
    
    
    /**
     * Returns the series protocol name of a series which is selected 
     * by "selectSeries".
     * @return protocol name
    */
    public native String getSeriesProtocolName ();
    
    
    /**
     * Returns the series operators name of a series which is selected 
     * by "selectSeries".
     * @return operator name
    */
    public native String getSeriesOperatorsName ();


	/** 
	 * Saves a monochrome bitmap as a DICOM Secondary Capture image.     
	 * The bitmap must use one byte per pixel, left to right, top to bottom     
	 * order of the pixels. 0 is interpreted as black, 255 as white.     
	 * @param filename the file name or path under which the image is saved.     
	 * @param pixelData a pointer to the image data. Must contain at least     
	 *   width*height bytes of data.     
	 * @param width the width of the image, must be <= 0xFFFF     
	 * @param height the height of the image, must be <= 0xFFFF     
	 * @param aspectRatio the pixel aspect ratio as width/height. If omitted, a pixel    
	 *   aspect ratio of 1/1 is assumed (default is 1.0).
	 * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
	*/    
	public native int saveDICOMImage(String filename, byte[] pixelData, long width, 
	                  long height, double aspectRatio);
	                  
	                  
    /**      
     * Resets the presentation state object to the status     
     * it had immediately after the last successful operation of "loadImage" or "loadPState".
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int resetPresentationState();
    
    
    /** 
     * Searches in the database for a DICOM instance with the given
     * study, series and instance UIDs and returns its pathname if found.
     * If the given instance is not found in the database, NULL is returned.
     * This method may only be called when the database is locked.
     * @param studyUID the DICOM study instance UID
     * @param seriesUID the DICOM series instance UID
     * @param instanceUID the DICOM SOP instance UID
     * @return filename (path) if found, NULL otherwise
    */
    public native String getFilename(String studyUID, String seriesUID, String instanceUID);
    
    
    /** 
     * Returns the Patient's Name of a study which is selected by        
     * the method  selectStudy 
    */    
    public native String getPatientName();
    
    
    /** 
     * Returns the PatientID of a study which is selected by        
     * the method  selectStudy 
    */   
    public native String getPatientID();
    
    
    /** 
     * Returns the PatientBirthDate of a study which is selected by       
     * the method  selectStudy 
    */    
    public native String getPatientBirthDate();
    
    
    /** 
     * Returns the PatientSex of a study which is selected by        
     * the method  selectStudy 
    */   
    public native String getPatientSex();
    
    
    /** 
     * Returns the PatientBirthTime of a study which is selected by
     * the method  selectStudy 
    */    
    public native String getPatientBirthTime(); 
    
    
    /** 
     * Returns the Other Patient Names of a study which is selected by        
     * the method  selectStudy 
    */    
    public native String getOtherPatientNames();
    
    
    /** 
     * Returns the OtherPatientID of a study which is selected by        
     * the method  selectStudy 
    */    
    public native String getOtherPatientID();
    
    
    /** 
     * Returns the PatientEthnicGroup of a study which is selected by        
     * the method  selectStudy 
    */    
    public native String getEthnicGroup();
    
    
    /** 
     * Returns the image number of an instance which is selected by the methods        
     * selectStudy, selectSeries and selectInstance 
    */
    public native String getImageNumber();
     
     
    /** 
     * Sends a complete study over network to a different DICOM peer.     
     * A separate application or process is launched to handle the send operation.     
     * This call returns when the send operation has successfully been launched.     
     * No information about the status or success of the transfer itself is being made    
     * available.     
     * @param targetID symbolic identifier of the send target, must be one of the   
     *     strings returned by getTargetID().     
     * @param studyUID Study Instance UID of the study to be sent. Must be a study    
     *     contained in the database.     
     * @return EC_Normal when the send process has successfully been launched,     
     *     an error condition otherwise (from jE_Condition).     
    */
    public native int sendStudy(String targetID, String studyUID);
    
    
    /** 
     * Sends a complete series over network to a different DICOM peer.     
     * A separate application or process is launched to handle the send operation.     
     * This call returns when the send operation has successfully been launched.     
     * No information about the status or success of the transfer itself is being made     
     * available.     
     * @param targetID symbolic identifier of the send target, must be one of the     
     *     strings returned by getTargetID().     
     * @param studyUID Study Instance UID of the series to be sent. Must be a series     
     *     contained in the database.     
     * @param seriesUID Series Instance UID of the series to be sent. Must be a series     
     *     contained in the database.     
     * @return EC_Normal when the send process has successfully been launched,     
     *     an error condition otherwise (from jE_Condition).
    */    
    public native int sendSeries(String targetID, String studyUID, String seriesUID);
    
    
    /** 
     * Returns the HOSTNAME entry for the communication partner with the given     
     * target ID from the configuration file.      
     * @param targetID communication target ID, must be one of the target      
     *   identifiers returned by getTargetID().     
     * @return entry if present in the config file, NULL otherwise.     
    */    
    public native String getTargetHostname(String targetID);    
    
    
    /** 
     * Returns the PORT entry for the communication partner with the given     
     * target ID from the configuration file.      
     * @param targetID communication target ID, must be one of the target      
     *   identifiers returned by getTargetID().     
     * @return entry if present and parsable in the config file, 0.     
    */    
    public native int getTargetPort(String targetID);    
    
    
    /** 
     * Returns the HOSTNAME entry for the communication partner with the given     
     * target ID from the configuration file.      
     * @param targetID communication target ID, must be one of the target      
     *   identifiers returned by getTargetID().     
     * @return entry if present in the config file, NULL otherwise.     
    */
    public native String getTargetAETitle(String targetID);    
    
    
    /** 
     * Returns the MAXPDU entry for the communication partner with the given     
     * target ID from the configuration file.      
     * @param targetID communication target ID, must be one of the target      
     *   identifiers returned by getTargetID().     
     * @return entry if present and parsable in the config file, 0.     
    */
    public native int getTargetMaxPDU(String targetID);    
    
    
    /** 
     * Returns the IMPLICITONLY entry for the communication partner with the given     
     * target ID from the configuration file.      
     * @param targetID communication target ID, must be one of the target      
     *   identifiers returned by getTargetID().     
     * @return entry if present in the config file, OFFalse otherwise.     
    */   
    public native boolean getTargetImplicitOnly(String targetID);    
    
    
    /** 
     * Returns the DISABLENEWVRS entry for the communication partner with the given     
     * target ID from the configuration file.      
     * @param targetID communication target ID, must be one of the target      
     *   identifiers returned by getTargetID().     
     * @return entry if present in the config file, OFFalse otherwise.     
    */
    public native boolean getTargetDisableNewVRs(String targetID);    
    
    
    /** 
     * Returns the database folder to be used for sending/receiving/browsing.     
     * Value is taken from the section GENERAL/DATABASE/DIRECTORY     
     * in the config file. If absent, a default value is returned.     
     * @return database folder path. Never returns NULL.     
    */    
    public native String getDatabaseFolder();    
    
    
    /** 
     * Returns the filename (path) of the DICOM Store SCU application used     
     * for sending images, as configured in section     
     * GENERAL/NETWORK/SENDER in the config file.     
     * @return send application path name or NULL if absent.     
    */    
    public native String getSenderName();    
    
    
    /** 
     * Returns the filename (path) of the DICOM Store SCP application used     
     * for receiving images, as configured in section     
     * GENERAL/NETWORK/RECEIVER in the config file.     
     * @return receive application path name or NULL if absent.     
    */
    public native String getReceiverName();
    
    
    /**
     * Create exclusiv access to the database
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int lockDatabase(boolean exclusive);
    
    
    /** 
     * Starts the network receiver process (Storage Service Class SCP).     
     * The receiver process will wait for incoming DICOM associations, receive images     
     * and presentation states. Data will be stored in file and registered in the     
     * database index file. Attention: Successful return of this method is no guarantee     
     * that the network receiver has successfully started, because certain errors     
     * (i.e. incorrect settings in the config file) will only be noted in the network     
     * receiver process when running. On Unix platform, successful return of this method     
     * means that the fork() used to start the receiver was successful.     
     * On Win32 platforms, it means that the CreateProcess() call was successful.     
     * @return EC_Normal if the receiver process could be started, an error code 
     *         otherwise (from jE_Condition).     
    */
    public native int startReceiver();
    
    
    /** 
     * Terminates the network receiver process (Storage Service Class SCP).     
     * This method attempts to terminate the network receiver process by     
     * requesting a DICOM association with it and delivering a special "shutdown" command.     
     * If for some reason the network receiver cannot be found (i.e. because it has     
     * terminated abnormally), a TCP/IP timeout (several seconds) may occur before this 
     * method returns.     
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */    
    public native int terminateReceiver();    
    
    
    /** 
     * Tests whether new DICOM objects have been received and     
     * added to the database since the last call to this method.     
     * @param true if new instances have been received.
    */    
    public native boolean newInstancesReceived();

    
    /** 
     * Returns the AETitle with which this application should identify itself.     
     * The AETitle is taken from the section GENERAL/NETWORK in the     
     * config file. If absent, a default value is returned.     
     * @return AETitle for this application. Never returns NULL.     
    */  
    public native String getNetworkAETitle();    
     
     
    /**
     * Returns the IMPLICITONLY entry for the network receiver     
     * from the section GENERAL/NETWORK in the config file.    
     * @return entry if present in the config file, OFFalse otherwise.     
    */ 
    public native boolean getNetworkImplicitVROnly();    
    
        
    /** 
     * Returns the DISABLENEWVRS entry for the network receiver    
     * from the section GENERAL/NETWORK in the config file.    
     * @return entry if present in the config file, OFFalse otherwise.   
    */
    public native boolean getNetworkDisableNewVRs();   
    
    
    /** 
     * Returns the BITPRESERVINGMODE entry for the network receiver   
     * from the section GENERAL/NETWORK in the config file.    
     * @return entry if present in the config file, OFFalse otherwise.     
    */ 
    public native boolean getNetworkBitPreserving();    
    
    
    /** 
     * Returns the PORT entry for the network receiver    
     * from the section GENERAL/NETWORK in the config file.     
     * @return entry if present and parsable in the config file, 0 otherwise.    
    */    
    public native int getNetworkPort();   
    

    /** 
     * Returns the MAXPDU entry for the network receiver     
     * from the section GENERAL/NETWORK in the config file.    
     * @return entry if present and parsable in the config file, 0 otherwise.  
    */ 
    public native int getNetworkMaxPDU();
    
    
    /** 
     * Returns the value of configuration file entry key=value     
     * in the section GENERAL/GUI of the config file.     
     * If the entry is absent, NULL is returned.     
     * @param key the entry key     
     * @return entry value or NULL.     
    */    
    public native String getGUIConfigEntry(String key);    
    
    
    /** 
     * Returns the value of configuration file entry key=value    
     * in the section GENERAL/GUI of the config file.   
     * If the entry is absent or cannot be parsed, the default dfl     
     * is returned.   
     * @param key the entry key    
     * @param dfl the default to be used if the value is absent or incorrect     
     * @return entry value or default.    
    */    
    public native boolean getGUIConfigEntryBool(String key, boolean dfl);
    
    
    /** 
     * Returns the filename (path) of the monitor characteristics file
     * used to implement the Barten transform, as configured in section
     * GENERAL/MONITOR/CHARACTERICSTICS in the config file.
     * @return monitor characteristics path name or null
    */
    public native String getMonitorCharacteristicsFile ();
                  
	                  
    /** 
     * Saves the DICOM image that is currently attached to the presentation state
     * in a file with the given path and filename.
     * This method does not acquire a database lock and does not register
     * the saved presentation state in the database.
     * @param filename path and filename under which the image is to be saved
     * @param explicitVR selects the transfer syntax to be written. True (the default) selects
     *   Explicit VR Little Endian, False selects Implicit VR Little Endian.
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
     */
    public native int saveCurrentImage(String filename, boolean explicitVR);

    
    /** saves a monochrome bitmap as a DICOM Secondary Capture image.
     *  The bitmap must use one byte per pixel, left to right, top to bottom
     *  order of the pixels. 0 is interpreted as black, 255 as white.
     *  @param filename the file name or path under which the image is saved.
     *  @param pixelData a pointer to the image data. Must contain at least
     *    width*height bytes of data.
     *  @param width the width of the image, must be <= 0xFFFF
     *  @param height the height of the image, must be <= 0xFFFF
     *  @param aspectRatio the pixel aspect ratio as width/height. If omitted, a pixel
     *    aspect ratio of 1/1 is assumed.
     *  @param explicitVR selects the transfer syntax to be written. 
     *    True selects Explicit VR Little Endian, False selects Implicit VR Little Endian.
     *  @param instanceUID optional parameter containing the SOP Instance UID to be written.
     *    This parameter should be omitted unless the SOP Instance UID needs to be controlled
     *    externally.
     *  @return EC_Normal upon success, an error code otherwise (from jE_Condition).
     */
    public native int saveDICOMImage(String filename, byte[] pixelData, long width,
            long height, double aspectRatio, boolean explicitVR, String instanceUID);
            
            
    /** 
     * Stores the current presentation state in a temporary place     
     * and creates a new presentation state that corresponds with an     
     * image displayed "without" presentation state.     
     * If called twice, an error code is returned.     
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).
    */
    public native int disablePState();    
     
     
    /** 
     * Restores the stored presentation state (see disablePresentationState)     
     * and deletes the temporary presentation state.     
     * If no stored presentation state exists, returns an error.     
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).     
    */    
    public native int enablePState();
    
    
    /** 
     * Returns number of presentation states referencing the currently selected image.     
     * If no instance is currently selected or the selected instance is a presentation     
     * state, returns an error.     
     * @return number of presentation states, 0 if none available or an error occurred     
    */    
    public native int getNumberOfPStates();    
    
    
    /** 
     * Selects and loads specified presentation state referencing the currently selected     
     * image.     
     * @param idx index to be selected, must be < getNumberOfPStates()     
     * @param changeStatus if true the presentation state is marked 'reviewed' (not new)
     * @return EC_Normal upon success, an error code otherwise (from jE_Condition).     
    */    
    public native int selectPState(int idx, boolean changeStatus);    
    
    
    /** 
     * Returns description of specified presentation state  referencing the currently     
     * selected image.     
     * @param idx index to be selected, must be < getNumberOfPStates()    
     * @return presentation state description or NULL if idx is invalid.
    */    
    public native String getPStateDescription(int idx);
    
    
    /** 
     * Checks whether Barten correction is possible, i.e.     
     * a valid monitor characteristics description exists     
     * and current system is a low-cost system (without built-in     
     * calibration).     
    */    
    public native boolean isBartenTransformPossible();    
    
    
    /** 
     * Sets ambient light value for the Barten transformation.     
     * @param value ambient light value to be set     
     * @return EC_Normal upon success, an error code otherwise(from jE_Condition).     
    */    
    public native int setAmbientLightValue(double value);        
    
    
    /** 
     * Returns ambient light value for the Barten transformation.     
     * @param value returned ambient light value. The object of class
     *        jDoubleByRef must be created before passing to this method!
     * @return EC_Normal upon success, an error code otherwise(from jE_Condition).     
    */    
    public native int getAmbientLightValue(jDoubleByRef value);
    
    
    /** 
     * Returns the Presentation Description of the currently selected instance.
     * May be called only if a valid instance selection exists - see selectInstance().
     * This method acquires a database lock which must be explicitly freed by the user.
     * @return Presentation Description or NULL if absent or not selected.
    */
    public native String getPresentationDescription();
    
    
    /** 
     * Returns label of specified presentation state referencing the 
     * currently selected image.     
     * @param idx index to be selected, must be < getNumberOfPStates()     
     * @return presentation state label or NULL idx is invalid     
    */    
    public native String getPStateLabel(int idx);
     
    
    /** 
     * Returns the Presentation Label of the currently selected instance.     
     * May be called only if a valid instance selection exists - see 
     * selectInstance().     
     * This method acquires a database lock which must be explicitly 
     * freed by the user.     
     * @return Presentation Label or NULL if absent or not selected.     
    */    
    public native String getPresentationLabel();
}
/*
 *  CVS Log
 *  $Log: jDVInterface.java,v $
 *  Revision 1.1.1.1  1999/05/14 09:33:09  Schroeter
 *  initial commit of release 1.0
 *
*/
