/*
 *
 *  Copyright (C) 1999, Institute for MicroTherapy
 *
 *  This software and supporting documentation were developed by
 *
 *    University of Witten/Herdecke
 *    Department of Radiology and MicroTherapy
 *    Institute for MicroTherapy
 *    Medical computer science
 *    
 *    Universitaetsstrasse 142
 *    44799 Bochum, Germany
 *    
 *    http://www.microtherapy.de/go/cs
 *    mailto:computer.science@microtherapy.de
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND THE INSTITUTE MAKES  NO 
 *  WARRANTY REGARDING THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY
 *  OR FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES 
 *  OR ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY 
 *  AND PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Author :      $Author: Schroeter $
 *  Last update : $Date: 1999/05/14 09:33:10 $
 *  Revision :    $Revision: 1.1.1.1 $
 *  State:        $State: Exp $
*/

package Viewer.Controller;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import Viewer.GUI.*;
import Viewer.Main.*;
import Viewer.Presentation.*;

/**
 * This class contains the listeners for the width PanelSlider 
 * in the WLTabPanel.
 * 
 * @author Klaus Kleber
 * @since 30.04.1999
 * @see GUI.WLTabPanel
 */
public class WLTabPanelWidthEventController implements SliderPanelListener
{
	/**
	 * Contains maximum scroll value.
	 * 
	 * @since 30.04.1999
	 */
    public int max =0;
    
	/**
	 * Contains minimum scroll value.
	 * 
	 * @since 30.04.1999
	 */
    public int min = 0;
    
	/**
	 * Contains text of the PanelSlider before changing. If an error occurs
	 * the text can be restore.
	 * 
	 * @since 30.04.1999
	 */
    String oldText;
	
	/**
	 * Contains the PanelSlider which fired the Events.
	 * 
	 * @since 30.04.1999
	 */
    PanelSlider panelSlider;
	
	
	/**
	 * Contains the current ScreenImageHandler
	 * 
	 * @since 30.04.1999
	 */
    ScreenImageHandler screenImageHandler;
   
	/**
	 * Constructs a new object.
	 * 
	 * @param screenImageHandler Contains the current screenImageHandler.
	 * @since 30.04.1999
	 */
    public WLTabPanelWidthEventController( ScreenImageHandler screenImageHandler)
    {
        this.screenImageHandler = screenImageHandler;
        oldText = new String("");
    }
    
	/**
	 * Sets a new PanelSlider.
	 * 
	 * @param panelSlider The new PanelSlider
	 * @since 30.04.1999
	 */
    public void setPanelSlider(PanelSlider panelSlider)
    {
        this.panelSlider = panelSlider;
    }
    
	/**
	 * Handles the ActionEvents fired from the PanelSlider. The ActionEvent 
	 * are fired from the TextField and results in a new width value.
	 * 
	 * @param e ActionEvent
	 * @since 30.04.1999
	 */
    public void actionPerformed(ActionEvent e)
    {
        //Aufrufendes Textfeld
        TextField text = (TextField)e.getSource();
        
        //Sperren des Listeners
        text.removeActionListener(this);
        setValue(text);
        text.addActionListener(this);
        
    }
    
	/**
	 * Handles the focus gained action  from the PanelSlider. 
	 * This action results in saving the text off the fireing TextField.
	 * 
	 * @param e FocusEvent
	 * @since 30.04.1999
	 */
    public void focusGained(FocusEvent e)
    {
        TextField text = (TextField)e.getSource();
        text.removeFocusListener(this);
        oldText = new String(text.getText());
        text.addFocusListener(this);
        
    }
	
	
	/**
	 * Handles the focus gained action   from the PanelSlider. The FocusEvent 
	 * are fired from the TextField and results in a new width value.
	 * 
	 * @param e FocusEvent
	 * @since 30.04.1999
	 */
    public void focusLost(FocusEvent e)
    {
        TextField text = (TextField)e.getSource();
        text.removeFocusListener(this);
        
        setValue(text);
        text.addFocusListener(this);
        
    }
	
	
	/**
	 * Handles the AdjustmentEvent fired from the PanelSlider. The AdjustmentEvent 
	 * are fired from the Scrollbar and results in a new width value.
	 * 
	 * @param e AdjustmentEvent
	 * @since 30.04.1999
	 */
    public void adjustmentValueChanged(AdjustmentEvent e)
    {
        Scrollbar scroll;
        
            scroll = (Scrollbar)e.getSource();
            scroll.removeAdjustmentListener(this);
            
            screenImageHandler.setNewWidth(panelSlider.getValue());
            panelSlider.setText((new Double(panelSlider.getValue()).toString()));
            
            scroll.addAdjustmentListener(this);
    }
    /**
    * Gets the  value from the specified TextField and sets a new width value.
    * If the value in text is outside the minimum or the maximum value 
    * the old text in the Textfiled will be restored.
    *
    * @param text Specifies the TextFiled with the value.
    * @since 30.04.1999
    */
    public void setValue(TextField text)
    {
        
        double newValue;
        
        try
        {
            //Umwandelung der Eingabe in einen intWert
            newValue = (new Double(text.getText().trim())).doubleValue();
            
            if (!( newValue <= 0.01d))
            {
                if (!((newValue <min) || (newValue > max)))
                {
                        
                    panelSlider.setScrollbarValue((int)(newValue));
                        
                }
                if(newValue <min)panelSlider.setScrollbarValue(min);
                if(newValue >max)panelSlider.setScrollbarValue(max);
                screenImageHandler.setNewWidth(newValue);
                oldText = new String (text.getText());
                    
            }
            else
            {
                text.setText( oldText);
            } 
        }
        catch(NumberFormatException t)
        {
            System.out.println("Number Format Exception" + t);
            text.setText( oldText);
        }    
        
    }
    /**
    * Sets new minimum and maximum values.
    *
    * @param min The new minimum value.
    * @param max The new maximum value.
    * @since 30.04.1999
    */
    public void setMinMax(int min, int max)
    {
        this.min = min;
        this.max = max;
        
    }

}

/*
 *  CVS Log
 *  $Log: WLTabPanelWidthEventController.java,v $
 *  Revision 1.1.1.1  1999/05/14 09:33:10  Schroeter
 *  initial commit of release 1.0
 *
*/
