/*
 *
 *  Copyright (C) 1999, Institute for MicroTherapy
 *
 *  This software and supporting documentation were developed by
 *
 *    University of Witten/Herdecke
 *    Department of Radiology and MicroTherapy
 *    Institute for MicroTherapy
 *    Medical computer science
 *    
 *    Universitaetsstrasse 142
 *    44799 Bochum, Germany
 *    
 *    http://www.microtherapy.de/go/cs
 *    mailto:computer.science@microtherapy.de
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND THE INSTITUTE MAKES  NO 
 *  WARRANTY REGARDING THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY
 *  OR FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES 
 *  OR ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY 
 *  AND PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Author :      $Author: Schroeter $
 *  Last update : $Date: 1999/05/14 09:33:11 $
 *  Revision :    $Revision: 1.1.1.1 $
 *  State:        $State: Exp $
*/

package Viewer.Main;

import Main.*;
import Viewer.GUI.*;
import Viewer.Controller.*;
import Viewer.Paint.*;
import Viewer.Presentation.*;
import J2Ci.*;

import java.awt.font.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.util.*;

/**
 * This calls manages the the image viewing and processing part of 
 * DICOMScope. This class contains all important objects of the GUI
 * and the data level.
 * <BR>
 * 
 * @author Klaus Kleber
 * @since 30.04.1999
 */

public class ScreenImageHandler extends java.lang.Object
{

	/**
	 * Contains the current PresentationStateGraphicsHandler.
	 * 
	 * @since 30.04.1999
	 */
	public PresentationStateGraphicsHandler presentationStateGraphicsHandler;
	
    /**
     * Contains the current ImageCanvas. This is the image drawing part of the GUI.
     * 
     * @since 30.04.1999
     */
    public ImageCanvas imageCanvas;
	
	
    /**
     * Contains the main GUI object.
     * 
     * @since 30.04.1999
     */
    public MainImageViewerPanel mainImageViewerPanel;
	
	
	/**
	 * MouseListener and MouseMotionListener which handles the MouseEvents.
	 * 
	 * @since 30.04.1999
	 */
	public ImageCanvasMouseEventListener imageCanvasMouseEventListener = null;
    
    
    
    /**
     * Object which transforms the visible part of the image to device space
     * 
     * @since 30.04.1999
     */
    public TransformationConverter transformationConverter;
    
    /**
     * Contains the visible part of the image in device space.
     * 
     * @since 30.04.1999
     */ 
    BufferedImage bufferedScreenImage = null;

    /**
     * Contains the size of the display.
     * 
     * @since 30.04.1999
     */
    Dimension screenSize;
    

	/**
	 * Contains the current PaintStructure
	 * 
	 * @since 30.04.1999
	 */
	public PaintStructure paintStructure ;

    
    /**
     * Builds the screenImageHandler.
     * 
     * @param mainImageViewerPanel Contains the main GUI object
     * @param presentationStateGraphicsHandler Contains the current ScreenImageHandler.
     * @since 30.04.1999
     */
    public  ScreenImageHandler(MainImageViewerPanel mainImageViewerPanel, PresentationStateGraphicsHandler  presentationStateGraphicsHandler)
    {
        this.mainImageViewerPanel = mainImageViewerPanel;
        this.presentationStateGraphicsHandler= presentationStateGraphicsHandler;
        //Builds the new image in User Space.
        screenSize = new Dimension(0,0);
    }
    
    
    /**
     * This function draws the visible part of the image in the 
     * specified Graphics2D object.
     * 
     * @param g2 Specifies the Graphics2D object.
     * @since 30.04.1999
     */
    public void drawScreenImage(Graphics2D g2)
    {
        if (presentationStateGraphicsHandler.havePresentationState)
        {
            if (!screenSize.equals(imageCanvas.getSize()))
            {
                presentationStateGraphicsHandler.setScreenSize(imageCanvas.getSize());
                setNewImage();
            }
                
            //Drawing the bufferedImage 
            if (bufferedScreenImage != null)
            {
                System.out.println("update");
                
                g2.drawImage(bufferedScreenImage,0,0, imageCanvas);
                paintStructure.aff = transformationConverter.getTransformation();
                paintStructure.drawState(g2);
            }
	    }
	    
    
    }//drawScreenImage

	/**
	 * Resets the all objects of the ScreenImageHandler.
	 *
	 * @since 30.04.1999
	 */
    
	public void resetState()
	{
        //removes the active event listeners
        if (imageCanvasMouseEventListener != null)
        {
            imageCanvas.removeMouseListener(imageCanvasMouseEventListener);
            imageCanvas.removeMouseMotionListener(imageCanvasMouseEventListener);
               
        }   
	    imageCanvas.removePopupMenu();
	    
	    //set cursor to default
	    imageCanvas.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    
	    //resets the state of the paintStructure
	    if (paintStructure!=null)paintStructure.reset();
	    
	    //reset the state of the presentationStateGraphicsHandler
	    if (presentationStateGraphicsHandler!=null)presentationStateGraphicsHandler.reset();
	    
	    
	    mainImageViewerPanel.annotationPanel.setActiveButton(null);
	}

	/**
	 * Builds the visible part of the image in the bufferedScreenImage.
	 * 
	 * @param first if true builds the bufferedScreenImage at first time.
	 * @param loadNewPixels if ture - load the pixels form the c++ part
	 * @param newScrollbarValue if true sets the scrollbar properties of the PanleScollImage.
	 * @param resetState if true resets all objects to their initial state.
	 * @since 30.04.1999
	 */
	    
	public void buildImageBuffer(boolean first,boolean loadNewPixels,boolean newScrollbarValue, boolean resetState)
	{
        if (resetState)resetState();
        
        if (!imageCanvas.getSize().equals(new Dimension(0,0)))
        {
        
            
            
                
            if (! first) transformationConverter.drawInDeviceSpace(  bufferedScreenImage,
                                                            presentationStateGraphicsHandler.getZoomValue(),
                                                            loadNewPixels);
            else transformationConverter.drawInDeviceSpaceFirst(  bufferedScreenImage,
                                                        presentationStateGraphicsHandler.getZoomValue(),
                                                        loadNewPixels);
            
            
                
                
            presentationStateGraphicsHandler.drawPaintStructure(bufferedScreenImage,transformationConverter.getTransformation(), transformationConverter.getOverlayTransformation());
                
                
	        if (newScrollbarValue) mainImageViewerPanel.getPanelScrollImage().setNewScrollbarValues(     transformationConverter.getScollValueWidth(), 
	                                                    transformationConverter.getScollValueHeight(),
	                                                    transformationConverter.getScrollbarValue());
                
            imageCanvas.repaint();
        }
	    
	}

	
	
    /**
     * Sets the ImageCanvas.
     * 
     * @param imageCanvas New ImageCanas.
     * @since 30.04.1999
     */
    public void setImageCanvas(ImageCanvas imageCanvas)
    {
        this.imageCanvas = imageCanvas;
        
    }
    
    
    
    
    /**
     * Sets a MouseEventListenerEditAnnotation the the imageCanvas.
     * 
     * @since 30.04.1999
     */
    public void setEditAction()
    {
        resetState();        
        
        imageCanvasMouseEventListener = new ImageCanvasEditListener(this);
        imageCanvas.addMouseListener(imageCanvasMouseEventListener);
        imageCanvas.addMouseMotionListener(imageCanvasMouseEventListener);
        mainImageViewerPanel.annotationPanel.setActiveButton();
        
    }  
   /**
    * Initialize all obejcts for drawing a new Annotation. The paintStructure will be reset and
    * the ImageCanvasNewListener will be set.
    * 
    * @param theAnnotation The new Annotation.
    * @since 30.04.1999
    */
    public void setNewPaintObjectAction(PaintObject thePaintObject)
    {
        paintStructure.setCurrentObject(null,true);  
        paintStructure.deleteCurrentPaintObject();
        if (imageCanvasMouseEventListener != null)
        {
            imageCanvas.removeMouseListener(imageCanvasMouseEventListener);
            imageCanvas.removeMouseMotionListener(imageCanvasMouseEventListener);
               
        }   
        paintStructure.oldPaintObject = null;
        
        paintStructure.setCurrentPaintObject(thePaintObject);
        
        imageCanvasMouseEventListener = new ImageCanvasNewListener(this);
        imageCanvas.addMouseListener(imageCanvasMouseEventListener);
        imageCanvas.addMouseMotionListener(imageCanvasMouseEventListener);
          
    }//setNewAnnotationAction
    
	
	
    /**
     * Sets the image on a new position of the display.  
     * 
     * @param newHeight Specifies the new virtualTLHC.y of the image. 
     * @since 30.04.1999
     * @see TransformationConverter
     */
    public void setNewImagePartHeight(int newHeight)
    {
	    resetState();
        transformationConverter.setVirtualTHLCHeight(newHeight);
        transformationConverter.drawInDeviceSpace(bufferedScreenImage,false);
	    presentationStateGraphicsHandler.drawPaintStructure(bufferedScreenImage,transformationConverter.getTransformation(),transformationConverter.getOverlayTransformation());
	    mainImageViewerPanel.funktionBasisTabPanel.presentationStatePanel.setValue();
        imageCanvas.repaint();
    }

    /**
     * Sets the image on a new position of the display.  
     * 
     * @param newWidth Specifies the new virtualTLHC.x of the image. 
     * @since 30.04.1999
     * @see TransformationConverter
     */
    public void setNewImagePartWidth(int newWidth)
    {
        resetState();
        transformationConverter.setVirtualTHLCWidth(newWidth);
        transformationConverter.drawInDeviceSpace(bufferedScreenImage,false);
	    presentationStateGraphicsHandler.drawPaintStructure(bufferedScreenImage,transformationConverter.getTransformation(),transformationConverter.getOverlayTransformation());

	    mainImageViewerPanel.funktionBasisTabPanel.presentationStatePanel.setValue();
        imageCanvas.repaint();
    }


	/**
	 * Sets a new center value for the image and draws the image.
	 *
	 * @param center the new center value.
	 * @since 30.04.1999
	 */
	public void setNewCenter(double center)
	{
	    double actualWidth = 0;
	    //System.out.println("Screen bin ich im center *: " +center);
        if (presentationStateGraphicsHandler.ps.haveActiveVOIWindow())
        {
            jDoubleByRef widthValue = new jDoubleByRef();
            presentationStateGraphicsHandler.ps.getCurrentWindowWidth(widthValue);
            actualWidth = (widthValue.value);
            
        }   
        else if (!presentationStateGraphicsHandler.ps.haveActiveVOILUT())
        {
            jDoubleByRef minValue = new jDoubleByRef();
            jDoubleByRef maxValue = new jDoubleByRef();
            presentationStateGraphicsHandler.ps.getImageMinMaxPixelRange(minValue,maxValue);
            actualWidth = ((maxValue.value-minValue.value+1));
        }
	    setNewWindow(center,actualWidth );
	}
	
	/**
	 * Sets a new widht value for the image and draws the image.
	 *
	 * @param width the new width value.
	 * @since 30.04.1999
	 */
	public void setNewWidth(double width)
	{
	    double actualCenter =0;
	    //System.out.println("Screen bin ich im Width *: " +width);
        if (presentationStateGraphicsHandler.ps.haveActiveVOIWindow())
        {
            jDoubleByRef centerValue = new jDoubleByRef();
            presentationStateGraphicsHandler.ps.getCurrentWindowCenter(centerValue);
            actualCenter = (centerValue.value);
        }   
        else if (!presentationStateGraphicsHandler.ps.haveActiveVOILUT())
        {
            jDoubleByRef minValue = new jDoubleByRef();
            jDoubleByRef maxValue = new jDoubleByRef();
            presentationStateGraphicsHandler.ps.getImageMinMaxPixelRange(minValue,maxValue);
            actualCenter = ((minValue.value+(maxValue.value-minValue.value+1)/2));
        }
	    
	    setNewWindow(actualCenter, width);
	}
	
	/**
	 * Set a new window.
	 * 
	 * @param width The new window with.
	 * @param center The new window center.
	 * @since 30.04.1999
	 */
	public void setNewWindow(double center, double width)
	{
	    //Sets new window
	    int status = presentationStateGraphicsHandler.ps.setVOIWindow( center, width ,"");
 	    //System.out.println("status: " +status);
           
            jDoubleByRef widthValue = new jDoubleByRef();
            presentationStateGraphicsHandler.ps.getCurrentWindowWidth(widthValue);
            //System.out.println("width: " + (new Double(widthValue.value).toString()));
            
            jDoubleByRef centerValue = new jDoubleByRef();
            presentationStateGraphicsHandler.ps.getCurrentWindowCenter(centerValue);
            //System.out.println("center: " + (new Double(centerValue.value).toString()));
	    
        
	    buildImageBuffer(false,true,false,true);
	    mainImageViewerPanel.funktionBasisTabPanel.presentationStatePanel.setValue();
	 }
	
	/**
	 * If a new jDVPresentationState object is available, this methode updates all object.
	 * 
	 * @since 30.04.1999
	 */
    public void setNewPresentationState()
    {
        if (!imageCanvas.getSize().equals(new Dimension(0,0)))
        {
        
            presentationStateGraphicsHandler.setNewPresentationState(imageCanvas.getSize());            
            transformationConverter = new JavaTransformationConverter(presentationStateGraphicsHandler);            
            paintStructure = new PaintStructure(presentationStateGraphicsHandler);            
            paintStructure.setImageCanvas(imageCanvas);            
            setNewImage();
        }
        
    }
    
    /**
     * Initialize the image data struture for new image. Called if a new image width other 
     * dimensions than the old image should be displayed.
     * 
     * @since 30.04.1999
     */
    public boolean setNewImage()
    {
        System.out.println("Set new Image");
        DataBuffer dbb;

        int bandOffsets[] = {0};

        WritableRaster wr ;
        
        
        
        //initial immagebuffer 
        if (!screenSize.equals(imageCanvas.getSize()))
        {
            bufferedScreenImage = null;
            wr= null;
            dbb= null;
            System.gc();
                
            dbb = new DataBufferByte( new byte[imageCanvas.getSize().height*imageCanvas.getSize().width],imageCanvas.getSize().height*imageCanvas.getSize().width);
            wr = Raster.createInterleavedRaster(dbb,imageCanvas.getSize().width,imageCanvas.getSize().height,imageCanvas.getSize().width,1, bandOffsets,null);
           // bufferedScreenImage = new BufferedImage(MainContext.instance().getUsedColorModel(), wr, MainContext.instance().getUsedColorModel().isAlphaPremultiplied(), null);
            bufferedScreenImage = new BufferedImage(MainContext.instance().getIndexColorModel(), wr, MainContext.instance().getIndexColorModel().isAlphaPremultiplied(), null);
            screenSize = new Dimension(imageCanvas.getSize());
                            
        }
	    buildImageBuffer(true,true,true,true);
        mainImageViewerPanel.funktionBasisTabPanel.layerPanel.deactiveAnnotationListisVisible = false;
        
        mainImageViewerPanel.setValue();
            
            //Paints the image 
        return true;
    }//setNewImage

   
    /**
     * Stops creating a new Annoatation and 
     * transforms the Annotation to a PresentationStateGraphicObject.
     *
     * @since 30.04.1999
     * @see Controller.PopupMenuController
     */
   public void setStopNew()
   {
      if (paintStructure.getCurrentPaintObject().getStatus() >=PaintObject.STATUS_WORK)
      {
            
            paintStructure.setWorkingShape(null);
            paintStructure.setCurrentStatus(PaintObject.STATUS_STOP);
            paintStructure.setCurrentPaintObject( paintStructure.getCurrentPaintObject().getNewPaintObject());
            buildImageBuffer(false,false,false,true);
            mainImageViewerPanel.funktionBasisTabPanel.layerPanel.setValue();
        
      }
      else paintStructure.deleteCurrentPaintObject();
   
   }

    /**
     * Closes a new Annoatation and 
     * transforms the Annotation to a PresentationStateGraphicObject.
     *
     * @since 30.04.1999
     * @see Controller.PopupMenuController
     */
   
   public void setCloseNew()
   {
      if (paintStructure.getCurrentPaintObject().getStatus() >=PaintObject.STATUS_WORK)
      {
         paintStructure.setWorkingShape(null);
         paintStructure.getCurrentPaintObject().setNewPoint(paintStructure.getCurrentPaintObject().getCopyPointAt(0));
         paintStructure.setCurrentStatus(PaintObject.STATUS_STOP);
         if (!paintStructure.getCurrentPaintObject().isShutter)
         {
            paintStructure.setCurrentPaintObject(paintStructure.getCurrentPaintObject().getNewPaintObject());
            buildImageBuffer(false,false,false,true);
            mainImageViewerPanel.funktionBasisTabPanel.layerPanel.setValue();
            
         }
         else
         {
            boolean repaint = false ;
            if (presentationStateGraphicsHandler.shutterList.haveBitmapShutter()) repaint = true;
            
            paintStructure.setCurrentPaintObject(paintStructure.getCurrentPaintObject().getNewPaintObject());
            buildImageBuffer(false,repaint, false,true);
            mainImageViewerPanel.funktionBasisTabPanel.shutterPanel.setValue();
         }
        }
      
   }
    
    

   /**
    * Sets the specified PresentationStateLayerObject to the active PresentationStateLayerObject.
    *
    * @param index The  index of the PresentationStateLayerObject
    * @since 30.04.1999
    */
   public void setCurrentLayerObject(int index)
   {
      presentationStateGraphicsHandler.setCurrentLayerObjectIndex(index);
      paintStructure.setCurrentObject(presentationStateGraphicsHandler.getCurrentLayerObject(),true);  
   }

    /**
     * Deletes the active PresentationStateGraphicObject in the presentationStateGraphicsHandler.
     *
     * @since 30.04.1999
     * @see Controller.PopupMenuController
     * @see PresentationStateGraphicsHandler
     */
   public void deleteActivePresentationStateGraphicObject()
   {
      if (presentationStateGraphicsHandler.getCurrentLayerObjectIndex() != -1)
      {
        presentationStateGraphicsHandler.deleteCurrentObject();
	    buildImageBuffer(false,false,false,true);
        mainImageViewerPanel.setValue();
        
      }
        
      else paintStructure.deleteCurrentPaintObject();
      
      setEditAction(); 
   }
   
   /**
    * This function search for the first PresentationStateLayerObject which contains the specified point.
    * 
    * @param clickedPoint The specified point.
    * @since 30.04.1999
    */
   public void setClick(Point2D.Float clickedPoint)
   {
        
        //
        paintStructure.setMoveAnn( clickedPoint);
        
        presentationStateGraphicsHandler.containsObject(clickedPoint);
          
        paintStructure.setCurrentObject(presentationStateGraphicsHandler.getCurrentLayerObject(),true);  
        mainImageViewerPanel.funktionBasisTabPanel.layerPanel.setValue();
        mainImageViewerPanel.annotationPanel.setActiveButton();
   }

    /**
     * Sets a new Point to the current Annotation in the paintStructure.
     * 
     * @param newPoint Contains the new Point. 
     * @since 30.04.1999
     */
    public void setNewPoint(Point2D.Float newPoint)
    {
        
         if (!paintStructure.getCurrentPaintObject().isShutter)
         {
            paintStructure.setNewPoint(newPoint);
            if (paintStructure.getCurrentPaintObject().status == PaintObject.STATUS_NULL)
            {
                buildImageBuffer(false,false,false,true);
            }
            mainImageViewerPanel.funktionBasisTabPanel.shutterPanel.setValue();
            
         }
         else
         {
            boolean repaint = false ;
            if (presentationStateGraphicsHandler.shutterList.haveBitmapShutter()) repaint = true;
               
            paintStructure.setNewPoint(newPoint);
            
            if (paintStructure.getCurrentPaintObject().status == PaintObject.STATUS_NULL)
            {
                paintStructure.setCurrentPaintObject(null);
                buildImageBuffer(false,repaint, false,true);
                mainImageViewerPanel.funktionBasisTabPanel.shutterPanel.setValue();
            }
         }
        
    }
    
    /**
     * Sets an new presentationStateTextObject form the specified parameters.
     * 
     * @param text Contains the text of the new presentationStateTextObject.
     * @param isBoundingBox Contains true if the new presentationStateTextObject have only a bounding box and no anchor point.
     * @param isAnchor Contains true if the new presentationStateTextObject have only an anchor point and no  bounding box .
     * @param isboth Contains true if the new presentationStateTextObject have  an achor point and a  bounding box .
     * @param isAnchorDisplay Contains true if the anchor point is display relative.
     * @param isBoxDisplay Contains true if the bounding box is display relative.
     * @param isAnchorVisible Contains ture if the anchor point is visible.
     * @since 30.04.1999
     */
    public void setNewText( String text, 
                            boolean isBoundingBox, 
                            boolean isAnchor,
                            boolean isboth, 
                            boolean isAnchorDisplay, 
                            boolean isBoxDisplay,
                            boolean isAnchorVisible)
    {
        
       
        resetState();
        
        //creates a new presentationStateTextObject.
        presentationStateGraphicsHandler.setNewText(text,isBoundingBox,isAnchor,isboth,isAnchorDisplay,isBoxDisplay,isAnchorVisible);
        
        //set the new current Object
        paintStructure.currentObject = presentationStateGraphicsHandler.getCurrentLayerObject();
        
        //
        imageCanvasMouseEventListener = new ImageCanvasMoveTextListener(this);
        imageCanvas.addMouseListener(imageCanvasMouseEventListener);
        imageCanvas.addMouseMotionListener(imageCanvasMouseEventListener);
        
    }
    
    /**
     * Sets either an new anchor or a new bounding box to an existing PresentationStateTextObject and
     * deletes the old if exist.
     * 
     * @param anchor If true insert anchor point, if false insert bounding box.
     * @param isDisplayRelative If true the anchor point is display relative.
     * @since 30.04.1999
     */
    public void setNewTextPart(boolean anchor, boolean isDisplayRelative)
    {
       
        PresentationStateTextObject currentText =(PresentationStateTextObject) paintStructure.currentObject;
        int layerIndex = presentationStateGraphicsHandler.getCurrentLayerIndex();
        int textIndex =presentationStateGraphicsHandler.getCurrentLayerObjectIndex();
        currentText.firstDraw = false;
        if (anchor)
        {
            
            currentText.needAnchorPoint= true;
            currentText.needBox= false;
            if ((currentText.getAnchorPointAnnotationUnits() == 1)||(isDisplayRelative))currentText.neededAnchorIsDisplayRelative = true;
            else currentText.neededAnchorIsDisplayRelative = false;
                
            //removes the anchor point
            currentText.removeAnchorPoint();
            
        }
        else
        {
            currentText.needAnchorPoint= false;
            currentText.needBox= true;
            if ((currentText.getBoundingBoxAnnotationUnits() == 1)||(isDisplayRelative))currentText.neededBoxIsDisplayRelative = true;
            else currentText.neededBoxIsDisplayRelative = false;
            //removes hte actual bounding box
            currentText.removeBoundingBox();
        }
	    currentText.setActive ( false);
	    //builds the new screen
	    buildImageBuffer(false,false, false,true);
        //sets the actual state
        presentationStateGraphicsHandler.setCurrentLayerObjectIndex(textIndex);
        presentationStateGraphicsHandler.setCurrentLayerIndex(layerIndex);
        
        
        paintStructure.currentObject= currentText;  
        
        if (imageCanvasMouseEventListener != null)
        {
            imageCanvas.removeMouseListener(imageCanvasMouseEventListener);
            imageCanvas.removeMouseMotionListener(imageCanvasMouseEventListener);
        }    
        imageCanvasMouseEventListener = new ImageCanvasMoveTextListener(this);
        imageCanvas.addMouseListener(imageCanvasMouseEventListener);
        imageCanvas.addMouseMotionListener(imageCanvasMouseEventListener);
        
    }

    /**
     * Initializes the active PresentationStateGraphicObject in the 
     * presentationStateGraphicsHandler for moving. Sets the ImageCanvasMoveListener.
     *
     * @since 30.04.1999
     * @see Controller.PopupMenuController
     */
    public void startMoveAnnotation()
    {
        
        presentationStateGraphicsHandler.getCurrentLayerObject().setActive ( false);
        int layerIndex = presentationStateGraphicsHandler.getCurrentLayerIndex();
        int graphicIndex =presentationStateGraphicsHandler.getCurrentLayerObjectIndex();
            
        //removes hte actual bounding box
	    //builds the new screen
	    buildImageBuffer(false,false, false,true);
                
        //sets the actual state
        presentationStateGraphicsHandler.setCurrentLayerIndex(layerIndex);
        presentationStateGraphicsHandler.setCurrentLayerObjectIndex( graphicIndex) ;
        paintStructure.currentObject=presentationStateGraphicsHandler.getCurrentLayerObject();  
        
        if (imageCanvasMouseEventListener != null)
        {
            imageCanvas.removeMouseListener(imageCanvasMouseEventListener);
            imageCanvas.removeMouseMotionListener(imageCanvasMouseEventListener);
                       
        }   
                
        imageCanvasMouseEventListener = new ImageCanvasMoveListener(this);
        imageCanvas.addMouseListener(imageCanvasMouseEventListener);
        imageCanvas.addMouseMotionListener(imageCanvasMouseEventListener);
        
    }

    /**
     * Stop moving the active PresentationStateGraphicObject in the
     * presentationStateGraphicsHandler.      
     *
     * @since 30.04.1999
     * @see Controller.PopupMenuController
     */
    public void stopMoveAnnotation()
    {
	    presentationStateGraphicsHandler.getCurrentLayerObject().setActive ( true);
	    buildImageBuffer(false,false,false,true);
	    setEditAction();
        
    }
    
    
    /**
     * Sets an new text value to an existing PresentationStateTextObject.
     * 
     * @param text The new text value.
     * @since 30.04.1999
     */
    public void setNewTextValue( String text )
    {
        PresentationStateTextObject editText = (PresentationStateTextObject)presentationStateGraphicsHandler.getCurrentLayerObject();
        editText.setNewValues(text);
        if (editText.mustRedraw) buildImageBuffer(false,false, false,true);
        setEditAction();        
    }

    /**
     * Stop moving the active PresentationStateTextObject in the
     * presentationStateGraphicsHandler.      
     *
     * @since 30.04.1999
     * @see Controller.PopupMenuController
     */
    public void stopMoveText()
    {
        PresentationStateTextObject textObject =(PresentationStateTextObject)paintStructure.currentObject;
        if (textObject.needBox &&textObject.needAnchorPoint)
        {
            textObject.needBox = false;
        }
        else
        {
            if (imageCanvasMouseEventListener != null)
            {
                imageCanvas.removeMouseListener(imageCanvasMouseEventListener);
                imageCanvas.removeMouseMotionListener(imageCanvasMouseEventListener);
                   
            }   
	        textObject.setActive ( true);
            
            buildImageBuffer(false,false, false, true);
            
            mainImageViewerPanel.funktionBasisTabPanel.layerPanel.setValue();
            setEditAction();   
            
        }
        
    }

    /**
     * Returns the current ImageCanvas.
     * 
     * @return The current ImageCanvas.
     * @since 30.04.1999
     */
    public ImageCanvas getImageCanvas()
    {
        return imageCanvas;
    }
    /**
     * Returns the content of the Image Canvas.
     * 
     * @return Returns the content of the Image Canvas.
     * @since 30.04.1999
     */
    public BufferedImage getBufferedScreenImage()
    {
        return bufferedScreenImage;
    }
    
}

/*
 *  CVS Log
 *  $Log: ScreenImageHandler.java,v $
 *  Revision 1.1.1.1  1999/05/14 09:33:11  Schroeter
 *  initial commit of release 1.0
 *
*/
