/*
 *
 *  Copyright (C) 1999, Institute for MicroTherapy
 *
 *  This software and supporting documentation were developed by
 *
 *    University of Witten/Herdecke
 *    Department of Radiology and MicroTherapy
 *    Institute for MicroTherapy
 *    Medical computer science
 *    
 *    Universitaetsstrasse 142
 *    44799 Bochum, Germany
 *    
 *    http://www.microtherapy.de/go/cs
 *    mailto:computer.science@microtherapy.de
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND THE INSTITUTE MAKES  NO 
 *  WARRANTY REGARDING THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY
 *  OR FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES 
 *  OR ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY 
 *  AND PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Author :      $Author: Schroeter $
 *  Last update : $Date: 1999/05/17 08:10:30 $
 *  Revision :    $Revision: 1.3 $
 *  State:        $State: Exp $
*/

package Viewer.Presentation;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;

import Main.*;
import J2Ci.*;

/**
 * This class contains methods which draw the visible part of the image and the 
 * shutter in a BufferedImage. 
 * <br>
 * The virtualTLHC contains a value for 
 * positioning the  the image on the display. The maximum value of  virtualTLHC.x is
 * (width of the image + width of the display), the maximum value of virtualTLHC.y is
 * (height of the image+ height of the display).
 * If  (virtualTLHC.x < width of the display) and (virtualTLHC.y< height of the display)
 * the TLHC of the image will drawn on the point virtualTLHC on the display.
 * If (virtualTLHC.x > width of the display) and (virtualTLHC.y > height of the display)
 * the point (width of the display - virtualTLHC.x, height of the display -virtualTLHC.y)
 *  of the image will be drawn on the top left hand coner of the display.
 * <BR>
 *
 * @author Klaus Kleber
 * @since 30.04.1999
 * @see JavaTransformationConverter
 * @see C_TransformationConverter
 */
public abstract class TransformationConverter
{

    
    /**
     * Contains the transfomations of the PresentationStateLayerObjects
     * and the SutterList.
     * 
     * @since 30.04.1999
     */
    AffineTransform aff;
    
    /**
     * Contains the transfomations of the PresentationStateOverlayObjects
     * 
     * @since 30.04.1999
     */ 
     AffineTransform overlayTransform;
    
    /**
     * Zoom value.
     * 
     * @since 30.04.1999
     */
    double zoomValue;
    
    
    /**
     * Contains the dimension of the display.
     * 
     * @since 30.04.1999
     */
    Dimension displayDimension;
    
    
    /**
     * Contains the displaying top left hand coner of the image.
     * 
     * @since 30.04.1999
     */
    protected Point sourcePixelBRHC;
   
    /**
     * Contains the displaying bottom right hand coner of the image.
     * 
     * @since 30.04.1999
     */
    protected Point sourcePixelTLHC;
    
    /**
     * Contains the top left hand coner of the display where the sourcePixelBRHC
     * of the image must be displayed. 
     * 
     * @since 30.04.1999
     */
    protected Point displayPixelTLHC;
    
    /**
     * Contains the botton right hand coner of the display where the sourcePixelTLHC
     * of the image must be displayed. 
     * 
     * @since 30.04.1999
     */
    protected Point displayPixelBRHC;
    
    /**
     * Contains the virtual top left hand corner(TLHC). The virtualTLHC contains a value for 
     * positioning the  the image on the display. The maximum value of  virtualTLHC.x is
     * (width of the image + width of the display), the maximum value of virtualTLHC.y is
     * (height of the image+ height of the display).
     * If  (virtualTLHC.x < width of the display) and (virtualTLHC.y< height of the display)
     * the TLHC of the image will drawn on the virtualTLHC.
     * If (virtualTLHC.x > width of the display) and (virtualTLHC.y > height of the display)
     * the point (width of the display - virtualTLHC.x, height of the display -virtualTLHC.y)
     *  of the image will be drawn on the top left hand coner of the display. 
     * 
     * 
     * @since 30.04.1999
     */
    Point virtualTLHC;
    
    
    /**
     * Contains the current PresentationStateGraphicsHandler.
     * 
     * @since 30.04.1999
     */ 
    PresentationStateGraphicsHandler presentationStateGraphicsHandler;
    
    
    /**
     * Constructs the object with the specified  PresentationStateGraphicsHandler.
     * 
     * @param presentationStateGraphicsHandler Conatins the current PresentationStateGraphicsHandler.
     * @see PresentationStateGraphicsHandler
     * @since 30.04.1999
     */
    public TransformationConverter(PresentationStateGraphicsHandler presentationStateGraphicsHandler)
    {
        //init var
        this.presentationStateGraphicsHandler = presentationStateGraphicsHandler;
        
        
        //gets the part of the image which should be visible.
        //Default ist the full image
        sourcePixelTLHC = new Point(0,0);
        sourcePixelBRHC = new Point(presentationStateGraphicsHandler.getCurrentImageWidth(),presentationStateGraphicsHandler.getCurrentImageHeight());
        
        
        //Sets the TLHC of the display in which the image should be paint.
        //Default ist the top left hand coner of the image;
        displayPixelTLHC = new Point(0,0);
    
        //init var
        displayDimension = new Dimension(0,0);
        
        //
        virtualTLHC = new Point(0,0);
        
    }
    
    
    /**
     * Returns the maximum y-value for the virtualTLHC. 
     * 
     * @return  The maximum y-value for the virtualTLHC. 
     * @since 30.04.1999
     */
    public int getScollValueHeight()
    {
        return (presentationStateGraphicsHandler.getCurrentImageHeight()+displayDimension.height);
    }

    /**
     * Returns the maximum x-value for the virtualTLHC. 
     * 
     * @return  The maximum x-value for the virtualTLHC. 
     * @since 30.04.1999
     */
    
    public int getScollValueWidth()
    {
        return(presentationStateGraphicsHandler.getCurrentImageWidth()+displayDimension.width);
    }

    /**
     * Returns the current virtualTLHC. 
     * 
     * @return The current virtualTLHC
     * @since 30.04.1999
     */
    public Point getScrollbarValue()
    {
        return virtualTLHC;
    }

    /**
     * Sets a new x-value for the virtualTLHC
     * 
     * @param newWidth New x-value for the virtualTLHC
     * @since 30.04.1999
     */
    
    public void setVirtualTHLCWidth(int newWidth)
    {
        virtualTLHC.x = newWidth;
    }

    /**
     * Sets a new y-value for the virtualTLHC
     * 
     * @param newHeight New y-value for the virtualTLHC
     * @since 30.04.1999
     */
    public void setVirtualTHLCHeight(int newHeight)
    {
        virtualTLHC.y = newHeight;
    }
    
    
    /**
     * Draws the specified display area of the image after the zooming, scaling, rotation and flipping 
     * in the specified BufferedImage.
     * 
     * @param bufferedImage Specifies the BufferedImage in which the image will be drawn.
     * @param zoomValue Specifies the zoom value of the image.
     * @param newPixels false if the pixels must get from the c++ part of the code.
     * @since 30.04.1999
     */
    public void drawInDeviceSpaceFirst (BufferedImage bufferedImage,double zoomValue,boolean newPixels)
    {
        
        
        this.zoomValue = zoomValue;
            
        //gets the dimension of the screen
        displayDimension.height = bufferedImage.getRaster().getHeight();
        displayDimension.width = bufferedImage.getRaster().getWidth();
        
        //Maps the display area. The display area specified in the DICOM Supp 33
        //is relative to the image pixel without ratiotion and flipping.
        //In the code the display area is relative to the image after ratation and flipping.
            
        //get display area values
        int xTLHC = presentationStateGraphicsHandler.ps.getDisplayedAreaTLHC_x()-1;
        int yTLHC =presentationStateGraphicsHandler.ps.getDisplayedAreaTLHC_y()-1;
        int xBRHC =presentationStateGraphicsHandler.ps.getDisplayedAreaBRHC_x()-1 ;
        int yBRHC =presentationStateGraphicsHandler.ps.getDisplayedAreaBRHC_y()-1;
            
        //converts to points
        Point2D.Float tlhc = new Point2D.Float(xTLHC,yTLHC);
        Point2D.Float brhc = new Point2D.Float(xBRHC,yBRHC);
            
	    //Converts the points TLHC and BRHC of the display area
	    AffineTransform  affDisplay = calculateSpatialTransfomation();     //Calculate transformation for Annotations, Shutter usw.
        affDisplay.transform(tlhc,tlhc);            
        affDisplay.transform(brhc,brhc);            
            
        //rotation of the image
        int rot = presentationStateGraphicsHandler.getRotation();
            
        //help value
        float change;
            
        //changes x values if image is flipped
        if (rot == 0)
        {
            if (presentationStateGraphicsHandler.ps.getFlip())
            {
                change = tlhc.x;
                tlhc.x = brhc.x;
                brhc.x = change;
            }
                    
        }
        //changes x values if image is rotated 90 degree
        //if the image is rotated 90 degree and flipped 
        //the transfomation eliminates each other.
        else if (rot == 1)
        {
            if (!presentationStateGraphicsHandler.ps.getFlip())
            {
                change = tlhc.x;
                tlhc.x = brhc.x;
                brhc.x = change;
            }
        }
        //changes x and y values if image is rotated 180 degree
        //if the image is rotated 180 degree and flipped only 
        //the y values must be changed
        else if (rot == 2)
        {
                   
            if (!presentationStateGraphicsHandler.ps.getFlip())
            {
                    
                change = tlhc.x;
                tlhc.x = brhc.x;
                brhc.x = change;
                        
                change = tlhc.y;
                tlhc.y = brhc.y;
                brhc.y = change;
            }
            else
            {
                change = tlhc.y;
                tlhc.y = brhc.y;
                brhc.y = change;
                        
            }
                    
        }
        //changes y values if image is rotated 270 degree
        //if the image is rotated 180 degree and flipped
        //the x an the y values must be changed
        else if (rot == 3)
        {
                   
            if (!presentationStateGraphicsHandler.ps.getFlip())
            {
                   
                change = tlhc.y;
                tlhc.y = brhc.y;
                brhc.y = change;
            }
            else
            {
                change = tlhc.x;
                tlhc.x = brhc.x;
                brhc.x = change;
                        
                change = tlhc.y;
                tlhc.y = brhc.y;
                brhc.y = change;
                        
            }
                    
        }
            
        //calculates the virtualTLHC 
        if (tlhc.x >=1) virtualTLHC.x =displayDimension.width+(int)tlhc.x;
        else 
        {
            virtualTLHC.x =displayDimension.width+(int)Math.round(tlhc.x*(zoomValue*presentationStateGraphicsHandler.getScalingX()));
        }
        if (tlhc.y >=1) virtualTLHC.y =displayDimension.height +(int)tlhc.y;
        else 
        {
            virtualTLHC.y =displayDimension.height+(int)Math.round(tlhc.y*(zoomValue*presentationStateGraphicsHandler.getScalingY()));
        }
        
       
        //draws the image in the bufferedImage
        drawInDeviceSpace(bufferedImage,newPixels);
        
    }
    
    
    /**
     * Draws the image in the specified ImageBuffer with the specified zoom value
     * 
     * @param bufferedImage Specifies the BufferedImage in which the image will be drawn.
     * @param zoomValue Specifies the zoom value of the image.
     * @param newPixels false if the pixels must get from the c++ part of the code.
     * @since 30.04.1999
     */
    public void drawInDeviceSpace (BufferedImage bufferedImage,double zoomValue,boolean newPixels)
    {
        this.zoomValue = zoomValue;
        
        displayDimension.height = bufferedImage.getRaster().getHeight();
        displayDimension.width = bufferedImage.getRaster().getWidth();
        drawInDeviceSpace(bufferedImage,newPixels);
        
    }
    
    /**
     * Draws the image in the specified ImageBuffer.
     * 
     * @param bufferedImage Specifies the BufferedImage in which the image will be drawn.
     * @param newPixels false if the pixels must get from the c++ part of the code.
     * @since 30.04.1999
     */
    public  void drawInDeviceSpace(BufferedImage bufferedImage,boolean newPixels)
    {
        
        int yTLHC;
        int xTLHC;
        int yBRHC;
        int xBRHC;
        
        //Sets new screenRows
        if ((displayDimension.height!= bufferedImage.getRaster().getHeight())||
            (displayDimension.width!= bufferedImage.getRaster().getWidth()))
        {
            if (!displayDimension.equals(new Dimension(0,0)))
            {
                virtualTLHC.y = bufferedImage.getRaster().getHeight();
                virtualTLHC.x = bufferedImage.getRaster().getWidth();
            }
            displayDimension.height = bufferedImage.getRaster().getHeight();
            displayDimension.width = bufferedImage.getRaster().getWidth();
        }
            
            
        //
        if ((virtualTLHC.x - displayDimension.width) >= 0)
        {
            sourcePixelTLHC.x = virtualTLHC.x - displayDimension.width;
            displayPixelTLHC.x = 0;
            xTLHC = sourcePixelTLHC.x;
                    
        }
        else
        {
            sourcePixelTLHC.x = 0;
            displayPixelTLHC.x = displayDimension.width-virtualTLHC.x;
            xTLHC = -(int)Math.round(displayPixelTLHC.x/(zoomValue*presentationStateGraphicsHandler.getCurrentScalingX()));
        }
                
        if((virtualTLHC.y - displayDimension.height)>=0)
        {
            sourcePixelTLHC.y = virtualTLHC.y - displayDimension.height;
            displayPixelTLHC.y = 0;
            yTLHC =sourcePixelTLHC.y;
        }
        else
        {
            sourcePixelTLHC.y = 0;
            displayPixelTLHC.y = displayDimension.height-virtualTLHC.y;
            yTLHC = -(int)Math.round(displayPixelTLHC.y/(zoomValue*presentationStateGraphicsHandler.getCurrentScalingY()));
                    
        }
                
        //System.out.println(" sourcePixelTLHC.width: " +sourcePixelTLHC.x);
        //System.out.println(" screenDimension.width: " +screenDimension.width);
        //System.out.println(" displayPixelTLHC.width: " +displayPixelTLHC.x);
                
        //Calculates new visible BRHC of the image.
        sourcePixelBRHC = new Point (  (int)Math.round( sourcePixelTLHC.x +  ((displayDimension.width-displayPixelTLHC.x)/(zoomValue*presentationStateGraphicsHandler.getCurrentScalingX()))),
                                        (int) Math.round(sourcePixelTLHC.y +  ((displayDimension.height-displayPixelTLHC.y)/(zoomValue*presentationStateGraphicsHandler.getCurrentScalingY()))));
                
        //Calculates the new dispaly BRHC
        displayPixelBRHC =  new Point(displayDimension.width,displayDimension.height);
            
        yBRHC = yTLHC +(int)Math.round(displayDimension.height/(zoomValue*presentationStateGraphicsHandler.getCurrentScalingY()));
        xBRHC = xTLHC +(int)Math.round(displayDimension.width/(zoomValue*presentationStateGraphicsHandler.getCurrentScalingX()));
            
	    //transforms the display area    
	    AffineTransform  affDisplay = calculateSpatialTransfomation();     
            
        Point2D.Float brhcPoint = new Point2D.Float(xBRHC,yBRHC);
        brhcPoint = getInverseTransformedPoint(brhcPoint,affDisplay);
            
        Point2D.Float tlhcPoint = new Point2D.Float(xTLHC,yTLHC);
        tlhcPoint = getInverseTransformedPoint(tlhcPoint,affDisplay);
    /*
        System.out.println("Hallo calculate TLHC ");
        System.out.println(" yBRHC " +yBRHC);
        System.out.println(" xBRHC " +xBRHC);
        System.out.println(" xTLHC " +xTLHC);
        System.out.println(" yTLHC " +yTLHC);
        System.out.println(" brhcPoint " +brhcPoint);
        System.out.println(" tlhcPoint " +tlhcPoint);
        */           
            
        // 
        float change;
        //changes x values if image is flipped
        int rot = presentationStateGraphicsHandler.getRotation();
        if (rot ==0)
        {
            if (presentationStateGraphicsHandler.ps.getFlip())
            {
                change=tlhcPoint.x;
                tlhcPoint.x = brhcPoint.x;
                brhcPoint.x = change;
            }
                    
        }
        //changes y values if image is rotated 90 degree
        //if the image is rotated 90 degree and flipped 
        //the transfomation eliminates each other.
        else if (rot == 1)
        {
            if (!presentationStateGraphicsHandler.ps.getFlip())
            {
                change = tlhcPoint.y;
                tlhcPoint.y = brhcPoint.y;
                brhcPoint.y = change;
            }
               
        }
        //changes x and y values if image is rotated 180 degree
        //if the image is rotated 180 degree and flipped only 
        //the y values must be changed
        else if (rot == 2)
        {
            if (!presentationStateGraphicsHandler.ps.getFlip())
            {
                    
                change = tlhcPoint.y;
                tlhcPoint.y = brhcPoint.y;
                brhcPoint.y = change;
                            
                change = tlhcPoint.x;
                tlhcPoint.x = brhcPoint.x;
                brhcPoint.x = change;
            }
            else
            {
                change = tlhcPoint.y;
                tlhcPoint.y = brhcPoint.y;
                brhcPoint.y = change;
                        
            }
                    
        }
        //changes x values if image is rotated 270 degree
        //if the image is rotated 180 degree and flipped
        //the x an the y values must be changed
        else if (rot == 3)
        {
            if (!presentationStateGraphicsHandler.ps.getFlip())
            {
                    
                change = tlhcPoint.x;
                tlhcPoint.x = brhcPoint.x;
                brhcPoint.x = change;
            }
            else
            {
                change = tlhcPoint.y;
                tlhcPoint.y = brhcPoint.y;
                brhcPoint.y = change;
                            
                change = tlhcPoint.x;
                tlhcPoint.x = brhcPoint.x;
                brhcPoint.x = change;
                        
            }
                    
        }
                
        //TLHC is never negative
        float diff;
        if (tlhcPoint.x <=1)
        {
            diff = brhcPoint.x - tlhcPoint.x;
            tlhcPoint.x = 1;
            brhcPoint.x = tlhcPoint.x + diff;
        }
        if (tlhcPoint.y <=1) 
        {
            diff = brhcPoint.y - tlhcPoint.y;
            tlhcPoint.y = 1;
            brhcPoint.y = tlhcPoint.y + diff;
        }
               
                    
        xBRHC = (int)brhcPoint.x;
        yBRHC = (int)brhcPoint.y;
        xTLHC = (int)tlhcPoint.x;
        yTLHC = (int)tlhcPoint.y;
                
        
        //sets the new display area to the c++ part.
        presentationStateGraphicsHandler.ps.setDisplayedAreaBRHC(xBRHC, yBRHC);
        presentationStateGraphicsHandler.ps.setDisplayedAreaTLHC(xTLHC, yTLHC);
        System.out.println("Hallo neuer TLHC ");
        System.out.println(" yBRHC " +yBRHC);
        System.out.println(" xBRHC " +xBRHC);
        System.out.println(" xTLHC " +xTLHC);
        System.out.println(" yTLHC " +yTLHC);
        
        //Calculates the transformations 
        calculateTransformation();
        
        draw(bufferedImage,newPixels);
    }

    /**
     * Draws the image in the specified ImageBuffer.
     * 
     * @param bufferedImage Specifies the BufferedImage in which the image will be drawn.
     * @param newPixels false if the pixels must get from the c++ part of the code.
    */
    public abstract void draw(BufferedImage bufferedImage,boolean newPixels);

    /**
     * Calculates the applying transformation 
     * 
     * @since 30.04.1999
     */

    void calculateTransformation()
    {
	            
	        //Calculate transformation for Annotations, Shutter usw.
	        aff = new AffineTransform();
	        overlayTransform = new AffineTransform();    
	        //Offsets
	            
	        //horizontal offset
	        double x;
	            
	        //Vertical offset
	        double y ;
	            
	            
	        if (sourcePixelTLHC.x >0) x=(-sourcePixelTLHC.x)*zoomValue;
	        else x=displayPixelTLHC.x-zoomValue;
	            
	        if (sourcePixelTLHC.y >0) y= (-sourcePixelTLHC.y)*zoomValue;
	        else y=displayPixelTLHC.y-zoomValue;
       
	        aff.translate(x,y);
	        overlayTransform.translate(x,y);
	        //zooming
	       // aff.scale(zoomValue/presentationStateGraphicsHandler.getCurrentScalingY(),zoomValue/presentationStateGraphicsHandler.getCurrentScalingX());
            aff.scale(zoomValue, zoomValue);
            overlayTransform.scale(zoomValue,zoomValue);
            System.out.println(" scaling x: " + presentationStateGraphicsHandler.getCurrentScalingX());
            System.out.println(" scaling y: " + presentationStateGraphicsHandler.getCurrentScalingY());
            //flipping
            if (presentationStateGraphicsHandler.ps.getFlip())
            {
                aff.scale(-1d,1d);
	            aff.translate(-presentationStateGraphicsHandler.getCurrentImageWidth()*presentationStateGraphicsHandler.getCurrentScalingX(),0);
	        }
	            
            //rotation
            int rot = presentationStateGraphicsHandler.ps.getRotation();
           
                
            //if image height an image width are diffenent, there must set an offset.
            double diff =(presentationStateGraphicsHandler.getImageWidth()*presentationStateGraphicsHandler.getScalingX()-presentationStateGraphicsHandler.getImageHeight()*presentationStateGraphicsHandler.getScalingY())/2;
	        if (rot ==1) aff.translate(-diff,-diff);
	        else if (rot == 3)aff.translate(diff,diff);
	        
            aff.rotate( (rot*0.5)*Math.PI,
                        (presentationStateGraphicsHandler.getCurrentImageWidth()*presentationStateGraphicsHandler.getCurrentScalingX()/2d),
                        (presentationStateGraphicsHandler.getCurrentImageHeight()*presentationStateGraphicsHandler.getCurrentScalingY()/2d));
	                 
        
    }
    
    /**
     * Builds the AffineTransfomation which rotate and flip points.
     * 
     * @since 30.04.1999
     */
    AffineTransform calculateSpatialTransfomation()
    {
        AffineTransform returnTransform = new AffineTransform();
        
        //rotation
        int rot = presentationStateGraphicsHandler.ps.getRotation();
                
        //if image height an image width are diffenent, there must set an offset.
        double diff =(presentationStateGraphicsHandler.getImageWidth()-presentationStateGraphicsHandler.getImageHeight())/2;
	    if (rot ==1) returnTransform.translate(-diff,diff);
	    else if (rot == 3)returnTransform.translate(-diff,diff);
	                
        returnTransform.rotate( (presentationStateGraphicsHandler.ps.getRotation()*0.5)*Math.PI,
                    ((presentationStateGraphicsHandler.getImageWidth()-1)/2d),
                    ((presentationStateGraphicsHandler.getImageHeight()-1)/2d));
        
        
        
        if (presentationStateGraphicsHandler.ps.getFlip())
        {
            if ((rot == 0)||(rot == 2))
            {
                returnTransform.scale(-1d,1d);
	            returnTransform.translate(-presentationStateGraphicsHandler.getImageWidth()+1,0);
	        }
	        else if ((rot== 1)||(rot == 3))
	        {
                returnTransform.scale(1d,-1d);
	            returnTransform.translate(0,-presentationStateGraphicsHandler.getImageHeight()+1);
	        }
	    }
	    
        return returnTransform;
    }
    
    /**
     * Gets the inverse transfomation of an specified point 
     * and an specified AffineTransformation.
     * 
     * @param displayPoint The point which should be transformed.
     * @param transAff The transformation which will be applied.
     * @return The inverse transfomation of the displayPoint.
     * @since 30.04.1999
     */
    public Point2D.Float getInverseTransformedPoint(Point2D.Float displayPoint,AffineTransform transAff)
    {
        Point2D.Float dstPoint = new Point2D.Float();
        try
        {
            transAff.inverseTransform(displayPoint,dstPoint);
        }
        catch(NoninvertibleTransformException e)
        {
            System.out.println("NoninvertibleTransformException: " + e);
        }
        return dstPoint;
    }
    
    /**
     * Conts the values of the specified array.
     * 
     * @param colorArray Specifies the array.
     * @return The number of values in the array.
     * @since 30.04.1999
     */
    public static int[] countValues(byte[] colorArray)
    {
        int[] returnArray = new int [256];
        for (int i = 0; i < colorArray.length; i++)
        {
            returnArray[colorArray[i]&0xff] = 1;
        }
        for (int i = 0; i < 256; i++)
        {
            System.out.println(" das ist der Wert i: " + i + " wert: " + returnArray[i]);
        }
        return returnArray;
     }
    /**
     * Returns the transformation for the PresentationStateAnnotationObject
     * 
     * @return The transformation for the PresentationStateAnnotationObject.
     * @since 30.04.1999
     */
    public AffineTransform getTransformation()
    {
        return aff;
    }
    
    /**
     * Returns the transformation for the PresentationStateOverlayObject
     * 
     * @return The transformation for the PresentationStateOverlayObject.
     * @since 30.04.1999
     */
    public AffineTransform getOverlayTransformation()
    {
        return overlayTransform;
    }
}



/*
 *  CVS Log
 *  $Log: TransformationConverter.java,v $
 *  Revision 1.3  1999/05/17 08:10:30  Schroeter
 *  Correction of some bad log tags
 *
 *  Revision 1.2  1999/05/14 10:02:43  Schroeter
 *  Log-Tags for CVS corrected
 *
 *  Revision 1.1.1.1  1999/05/14 09:33:11  Schroeter
 *  initial commit of release 1.0
 *
*/
