/*
 *
 *  Copyright (C) 1999, Institute for MicroTherapy
 *
 *  This software and supporting documentation were developed by
 *
 *    University of Witten/Herdecke
 *    Department of Radiology and MicroTherapy
 *    Institute for MicroTherapy
 *    Medical computer science
 *    
 *    Universitaetsstrasse 142
 *    44799 Bochum, Germany
 *    
 *    http://www.microtherapy.de/go/cs
 *    mailto:computer.science@microtherapy.de
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND THE INSTITUTE MAKES  NO 
 *  WARRANTY REGARDING THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY
 *  OR FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES 
 *  OR ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY 
 *  AND PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Author :      $Author: Schroeter $
 *  Last update : $Date: 1999/05/14 09:33:10 $
 *  Revision :    $Revision: 1.1.1.1 $
 *  State:        $State: Exp $  
*/

package jToolkit.Gui;


import java.awt.*;
import java.awt.event.*;

/**
 * The CommandAWTScrollbar extends the normal Scrollbar with an automatic command
 * call when the button is pressed.
 * If flag "eventOnlyOnMouseRelease" is set (true), an events are only fired <delay> 
 * milli-secs after the last change of the scrollbar.
 *
 * Usage:
 * 
 * class FooClass extends Panel implements CommandScrollbarListener
 * {
 *  final int IDVERTSCR = 1;
 *
 *  void Constructor ()
 *  {
 *      CommandAWTScrollbar csb = new CommandAWTScrollbar (Scrollbar.VERTICAL, 0, 10, 0, 1010, this, IDVERTSCR);
 *		
 *      csb.setEventOnlyOnMouseRelease (true);
 * 		
 *		add (csb, "East");
 *  }
 * 
 *	public void ScrollbarChanged (int ID, int toValue)
 *	{
 *	    if (ID == IDVERTSCR) System.out.println ("Vertical scrollbar changed to " + toValue);
 *	}
 * } 
 *
 * @author Andreas Schroeter
 * @since 30.03.
*/
public class CommandAWTScrollbar extends Scrollbar implements AdjustmentListener, Runnable
{
    private int ID;
    private CommandScrollbarListener csbl;
    private long t1, t2;
    private boolean eventOnlyOnMouseRelease;
    private boolean eventFired;
    private int fireDelay = 100;  // msec    
        
    /**
     * Constructor 1 (see Constructor of java.awt.Scrollbar)
    */
    public CommandAWTScrollbar(int orientation, int value, int visible, int minimum, 
           int maximum, CommandScrollbarListener csbl, int ID)
    {
        super (orientation, value, visible, minimum, maximum);
        setParams (csbl, ID);
    }

    /**
     * Constructor 2 (see Constructor of java.awt.Scrollbar)
    */
    public CommandAWTScrollbar(int orientation, CommandScrollbarListener csbl, int ID)
    {
        super (orientation);
        setParams (csbl, ID);        
    }
        
    /**
     * Constructor 3 (see Constructor of java.awt.Scrollbar)
    */
    public CommandAWTScrollbar(CommandScrollbarListener csbl, int ID)
    {
        super ();        
        setParams (csbl, ID);        
    }
    
    private void setParams (CommandScrollbarListener csbl, int ID)
    {
        eventOnlyOnMouseRelease = false;
        eventFired = true;
        
        this.ID = ID;
	    this.csbl = csbl;
        addAdjustmentListener(this);
        
        t1 = t2 = 0;
        
        Thread t = new Thread (this);
        t.setPriority(Thread.MIN_PRIORITY);
        t.start();        
    }
    
    /**
     * Sets the flag for event-handling. If flag is false, this scrollbar will fire events 
     * like the normal scrollbar. If flag is true, an event will only be fired after some time
     * (= delay) after the last change of the scrollbar.
     * That means if the scrollbar is moved up and down fast, no event is fired until the
     * scrollbar-move is stopped (for "delay" millisecs at least).
     * @param flag the flag for event-handling.
    */
    public void setEventOnlyOnMouseRelease (boolean flag)
    {
        eventOnlyOnMouseRelease = flag;
    }
    
    /**
     * Sets the minimum time for waiting for the last scrollbar move. An event is fired
     * "delay" millisecs after the last scrollbar move (if event-flag is true).
     * Possible values are between 1 and 5000.
     * @param delay the delay time
    */
    public void setFireDelay (int delay)
    {
        if (delay > 0 && delay < 5000);
        fireDelay = delay;
    }
    
    /**
     * Thread-Method.
    */
    public void run ()
    {
        while (true)
        {
            t2 = System.currentTimeMillis();
            
            if ((t2 - t1 > fireDelay) && !eventFired)
            {
                eventFired = true;
                fireEvent();
            }
            
            try
	        {
	            Thread.sleep (25); // wait some secs
	        }
	        catch(Exception e)
	        {
	        }
        }
    }
    
    
    private void fireEvent ()
    {
        csbl.ScrollbarChanged (ID, getValue());
    }
    
    /**
     * internal Event-Handler.
    */
    public void adjustmentValueChanged(AdjustmentEvent e)
    {        
        if (e.getSource() != this) return;       
        
        if (eventOnlyOnMouseRelease)
        {
            t1 = System.currentTimeMillis();
            eventFired = false;
        }
        else csbl.ScrollbarChanged (ID, e.getValue());
    }
}
/*
 *  CVS Log
 *  $Log: CommandAWTScrollbar.java,v $
 *  Revision 1.1.1.1  1999/05/14 09:33:10  Schroeter
 *  initial commit of release 1.0
 *
*/
